/*
 * @Author: Dwt 
 * @Date: 2019-11-11 12:41:11 
 * @Last Modified by: Dwt
 * @Last Modified time: 2019-11-11 22:03:11
 */

import 'dart:io';

import 'package:camera/camera.dart';
import 'package:camera_custom/camera_custom.dart';
import 'package:flutter/material.dart';
import 'package:path_provider/path_provider.dart';

class CameraCustomPage extends StatefulWidget {
  //获取摄像头数组
  List<CameraDescription> cameras;
  CameraCustomPage(this.cameras);

  @override
  _CameraCustomState createState() => _CameraCustomState();
}

class _CameraCustomState extends State<CameraCustomPage> {
  //图片路径
  String imagePath;
  //是否切换摄像头
  bool _isToggleCamera = false;
  //摄像头控制器
  CameraController controller;

  @override
  void initState() {
    // TODO: implement initState

    try {
      //后置摄像头
      onCameraPicked(widget.cameras[0]);
    } catch (e) {
      print(e.toString());
    }
    super.initState();
  }

  @override
  void dispose() {
    // TODO: implement dispose
    controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (widget.cameras.isEmpty) {
      return Container(
        alignment: Alignment.center,
        padding: EdgeInsets.all(15.0),
        child: Text(
          '未检测到摄像头',
          style: TextStyle(fontSize: 15, color: Colors.white),
        ),
      );
    }

    if (!controller.value.isInitialized) {
      return Container();
    }

    return AspectRatio(
      ///使用摄像头比例
      aspectRatio: controller.value.aspectRatio,
      child: Container(
        child: Stack(
          children: <Widget>[
            //预览视图
            CameraPreview(controller),
            //底部操作视图
            setupBottomView(context),
          ],
        ),
      ),
    );
  }

  //使用摄像头
  void onCameraPicked(CameraDescription cameraDescription) async {
    //先释放掉原有控制器
    if (controller != null) await controller.dispose();
    controller = CameraController(cameraDescription, ResolutionPreset.high);

    //添加监听
    controller.addListener(() {
      if (mounted) {
        setState(() {});
      }
      if (controller.value.hasError) {
        showMsg('Camera Error: ${controller.value.errorDescription}');
      }
    });

    //初始化
    try {
      await controller.initialize();
    } on CameraException catch (e) {
      showException(e);
    }

    if (mounted) {
      setState(() {});
    }
  }

  //捕获视图
  _captureImage() {
    takePicture().then((String filePath) {
      if (mounted) {
        setState(() {
          imagePath = filePath;
        });

        if (filePath != null) {
          showMsg('图片存在$filePath');
          //返回
          popWithResult();
        }
      }
    });
  }

  //拍照
  Future<String> takePicture() async {
    if (!controller.value.isInitialized) {
      showMsg('请先选择摄像头');
      return null;
    }

    //图片路径缓存在文件夹中
    final Directory exDir = await getApplicationDocumentsDirectory();
    final String dirPath = '${exDir.path}/FlutterDevs/Camera/Images';
    await Directory(dirPath).create(recursive: true);
    final String filePath = '$dirPath/${timestamp()}.png';

    if (controller.value.isTakingPicture) {
      return null;
    }

    try {
      await controller.takePicture(filePath);
    } on CameraException catch (e) {
      showException(e);
      return null;
    }
    return filePath;
  }

  //构建底部视图
  Widget setupBottomView(BuildContext context) {
    return Align(
      alignment: Alignment.bottomCenter,
      child: Container(
        width: double.infinity,
        height: 120,
        padding: EdgeInsets.all(20.0),
        color: Color.fromRGBO(0, 0, 0, 0.7),
        child: Stack(
          children: <Widget>[
            //拍摄视图部分
            setupShootView(),
            //切换视图按钮
            setupSwitchView(context)
          ],
        ),
      ),
    );
  }

  //拍摄按钮部分
  Widget setupShootView() {
    return Align(
      alignment: Alignment.center,
      child: Material(
        color: Colors.transparent,
        child: InkWell(
          borderRadius: BorderRadius.all(Radius.circular(5)),
          onTap: () {
            _captureImage();
          },
          child: Container(
              width: 72,
              height: 72,
              padding: EdgeInsets.all(4.0),
              child: Icon(
                Icons.photo_camera,
                color: Colors.white,
              )
            ),
        ),
      ),
    );
  }

  //切换按钮部分
  Widget setupSwitchView(BuildContext context) {
    return Align(
      alignment: Alignment.centerRight,
      child: Material(
        color: Colors.transparent,
        child: InkWell(
          borderRadius: BorderRadius.all(Radius.circular(50)),
          onTap: () {
            if (!_isToggleCamera) {
              onCameraPicked(widget.cameras[1]);
              setState(() {
                _isToggleCamera = true;
              });
            } else {
              onCameraPicked(widget.cameras[0]);
              setState(() {
                _isToggleCamera = false;
              });
            }
          },
          child: Container(
              width: 72,
              height: 72,
              padding: EdgeInsets.all(4.0),
              child: Icon(
                Icons.photo_filter,
                color: Colors.white,
              )
            ),
        ),
      ),
    );
  }

  //时间戳
  String timestamp() {
    return DateTime.now().millisecondsSinceEpoch.toString();
  }

  //返回
  popWithResult() {
    Navigator.pop(context, imagePath);
  }

  void showMsg(String message) {
    print(message);
  }

  void showException(CameraException e) {
    logError(e.code, e.description);
    showMsg('Error:${e.code}\n${e.description}');
  }

  void logError(String code, String message) =>
      print('Error: $code\nMessage: $message');
}
